@@ -1,3 +1,6 @@
+extern "C" {
+#include "kmplayer.c"
+}
 #include "strings.cpp"
 #include "io.cpp"
 #include "log.cpp"
