@@ -31,29 +31,24 @@ using namespace KMPlayer;
 String NodeValue::value () const {
     if (attr)
         return attr->value ();
-    return node->nodeValue ();
+    if (node)
+        return node->nodeValue ();
+    return string;
 }
 
 namespace {
 
 struct EvalState {
     EvalState (EvalState *p, const String &root_tag=String())
-     : def_root_tag (root_tag), root (NULL), attr (NULL),
+     : def_root_tag (root_tag), root (NULL),
        process_list (NULL), parent (p),
        sequence (1), ref_count (0) {}
 
     void addRef () { ++ref_count; }
     void removeRef () { if (--ref_count == 0) delete this; }
 
-    String value () const {
-        if (attr)
-            return attr->value ();
-        return root->nodeValue ();
-    }
-
     String def_root_tag;
-    Node *root;
-    Attribute *attr;
+    NodeValue root;
     Sequence *process_list;
     EvalState *parent;
     int sequence;
@@ -75,7 +70,7 @@ struct AST : public Expression {
     virtual Sequence *toSequence () const; // uncachable
     virtual Type type () const;
     virtual void setRoot (Node *root);
-    void setRoot (Node *root, Attribute *a);
+    void setRoot (const NodeValue &value);
 #ifdef KMPLAYER_EXPR_DEBUG
     virtual void dump () const;
 #endif
@@ -151,19 +146,29 @@ struct StringBase : public AST {
     mutable String string;
 };
 
-struct Step : public StringBase {
-    Step (EvalState *ev, bool context=false)
-     : StringBase (ev),
-       any_node (false), context_node (context),
-       is_attr (false), start_contextual (false) {}
-    Step (EvalState *ev, const char *s, const char *e, bool isattr=false)
-     : StringBase (ev, s, e),
-       any_node (string == "*"), context_node (false),
-       is_attr (isattr), start_contextual (false) {}
+struct SequenceBase : public StringBase {
+    SequenceBase (EvalState *ev) : StringBase (ev) {}
+    SequenceBase (EvalState *ev, const char *s, const char *e)
+        : StringBase (ev, s, e) {}
 
-    bool matches (Node *n);
-    bool matches (Attribute *a);
-    bool anyPath () const { return string.isEmpty (); }
+    virtual bool toBool () const;
+    virtual String toString () const;
+    virtual Type type () const;
+};
+
+struct Step : public SequenceBase {
+    Step (EvalState *ev, bool context=false)
+     : SequenceBase (ev),
+       any_node (false), context_node (context),
+       is_attr (false), recursive (false) {}
+    Step (EvalState *ev, const char *s, const char *e, bool isattr, bool rec)
+     : SequenceBase (ev, s, e),
+       any_node (string == "*"), context_node (false),
+       is_attr (isattr), recursive (rec) {}
+
+    Sequence *toSequence () const;
+    bool matches (Node *n) const;
+    bool matches (Attribute *a) const;
 #ifdef KMPLAYER_EXPR_DEBUG
     virtual void dump () const {
         fprintf (stderr, "Step %c%s",
@@ -173,34 +178,44 @@ struct Step : public StringBase {
     }
 #endif
 
-    TrieString attr;
+    mutable TrieString attr;
     bool any_node;
     bool context_node;
     bool is_attr;
-    bool start_contextual;
-};
-
-struct SequenceBase : public StringBase {
-    SequenceBase (EvalState *ev) : StringBase (ev) {}
-
-    virtual bool toBool () const;
-    virtual String toString () const;
-    virtual Type type () const;
+    bool recursive;
 };
 
 struct Identifier : public SequenceBase {
-    Identifier (EvalState *ev, AST *steps) : SequenceBase (ev) {
+    Identifier (EvalState *ev, AST *steps, bool context)
+        : SequenceBase (ev), start_contextual (context) {
         first_child = steps;
     }
 
     virtual Sequence *toSequence () const;
-    void childByStep (Step *step, bool recurse) const;
-    void childByPath (Step *step, bool recurse) const;
 #ifdef KMPLAYER_EXPR_DEBUG
     virtual void dump () const {
         fprintf (stderr, "Identifier ");
         AST::dump();
     }
+#endif
+    bool start_contextual;
+};
+
+struct PredicateFilter : public SequenceBase {
+    PredicateFilter (EvalState *ev, AST *children) : SequenceBase (ev) {
+        first_child = children;
+    }
+
+    virtual Sequence *toSequence () const;
+#ifdef KMPLAYER_EXPR_DEBUG
+    virtual void dump () const {
+        fprintf (stderr, "Predicate ");
+        first_child->dump();
+        fprintf (stderr, "[");
+        for (AST *n = first_child->next_sibling; n; n = n->next_sibling)
+            n->dump();
+        fprintf (stderr, "]");
+    }
 #endif
 };
 
@@ -332,6 +347,18 @@ struct Sort : public SequenceBase {
     virtual Sequence *toSequence () const;
 };
 
+struct SubSequence : public SequenceBase {
+    SubSequence (EvalState *ev) : SequenceBase (ev) {}
+
+    virtual Sequence *toSequence () const;
+};
+
+struct Tokenize : public SequenceBase {
+    Tokenize (EvalState *ev) : SequenceBase (ev) {}
+
+    virtual Sequence *toSequence () const;
+};
+
 struct Multiply : public NumberBase {
     Multiply (EvalState *ev, AST *children) : NumberBase (ev) {
         first_child = children;
@@ -476,14 +503,12 @@ AST::Type AST::type () const {
 }
 
 void AST::setRoot (Node *root) {
-    eval_state->root = root;
-    eval_state->attr = NULL;
-    eval_state->sequence++;
+    setRoot (NodeValue (root));
 }
 
-void AST::setRoot (Node *root, Attribute *a) {
-    setRoot (root);
-    eval_state->attr = a;
+void AST::setRoot (const NodeValue& value) {
+    eval_state->root = value;
+    eval_state->sequence++;
 }
 
 #ifdef KMPLAYER_EXPR_DEBUG
@@ -536,8 +561,10 @@ bool NumberBase::toBool () const {
             int count = 0;
             for (NodeValueItem *n = lst->first (); n; n = n->nextSibling ())
                 if (ii == ++count)
-                    return eval_state->root == n->data.node &&
-                        eval_state->attr == n->data.attr;
+                    return eval_state->root.node
+                        ? eval_state->root.node == n->data.node &&
+                          eval_state->root.attr == n->data.attr
+                        : eval_state->root.value () == n->data.value ();
         }
         return false;
     }
@@ -588,11 +615,11 @@ AST::Type StringBase::type () const {
     return TString;
 }
 
-bool Step::matches (Node *n) {
+bool Step::matches (Node *n) const {
     return any_node || string == n->nodeName ();
 }
 
-bool Step::matches (Attribute *a) {
+bool Step::matches (Attribute *a) const {
     if (attr.isNull () && !string.isEmpty ())
         attr = (const char *) string;
     return any_node || attr == a->name ();
@@ -640,80 +667,86 @@ AST::Type SequenceBase::type () const {
     return TString;
 }
 
-void Identifier::childByStep (Step *step, bool recurse) const {
-    Sequence *lst = eval_state->process_list;
-    NodeValueItem *last = lst->last ();
+Sequence *Step::toSequence () const {
+    Sequence *lst = new Sequence;
+    NodeValueItem *last = eval_state->process_list->last ();
     Sequence recursive_lst;
-    for (NodeValueItem *itm = lst->first (); itm; ) {
+    for (NodeValueItem *itm = eval_state->process_list->first (); itm; ) {
         NodeValueItem *next = itm == last ? NULL : itm->nextSibling ();
 
         Node *n = itm->data.node;
-        if (step->is_attr) {
+        if (!n)
+            continue; // FIXME
+        if (is_attr) {
             Element *e = n->isElementNode() ? static_cast<Element *> (n) : NULL;
             Attribute *a = e ? e->attributes ().first () : NULL;
             for (; a; a = a->nextSibling ())
-                if (step->matches (a))
+                if (matches (a))
                     lst->append (new NodeValueItem (NodeValue (n, a)));
-            if (recurse)
+            if (recursive)
                 for (Node *c = n->firstChild(); c; c = c->nextSibling ())
                     recursive_lst.append (new NodeValueItem (c));
-        } else if (step->context_node) {
+        } else if (context_node) {
             if (eval_state->parent)
-                lst->append (new NodeValueItem (eval_state->parent->root));
+                lst->append (new NodeValueItem (eval_state->root));
         } else {
             for (Node *c = n->firstChild(); c; c = c->nextSibling ()) {
-                if (step->matches (c))
+                if (matches (c))
                     lst->append (new NodeValueItem (c));
-                if (recurse)
+                if (recursive)
                     recursive_lst.append (new NodeValueItem (c));
             }
         }
 
-        lst->remove (itm);
         itm = next;
     }
-    if (recurse && recursive_lst.first ()) {
+    if (recursive && recursive_lst.first ()) {
+        Sequence *old = eval_state->process_list;
         eval_state->process_list = &recursive_lst;
-        childByStep (step, recurse);
-        for (NodeValueItem *r = recursive_lst.first (); r; r = r->nextSibling ())
+        Sequence *rlst = toSequence ();
+        for (NodeValueItem *r = rlst->first (); r; r = r->nextSibling ())
             lst->append (new NodeValueItem (r->data));
-        eval_state->process_list = lst;
+        delete rlst;
+        eval_state->process_list = old;
     }
-}
-
-void Identifier::childByPath (Step *step, bool recurse) const {
-    if (!step->anyPath ()) {
-        childByStep (step, recurse);
-        for (AST *pred = step->first_child; pred; pred = pred->next_sibling) {
-            NodeValueItem *itm = eval_state->process_list->first ();
-            if (itm) {
-                Sequence *newlist = new Sequence;
-                for (; itm; itm = itm->nextSibling ()) {
-                    pred->setRoot (itm->data.node, itm->data.attr);
-                    if (pred->toBool ())
-                        newlist->append (new NodeValueItem (itm->data));
-                }
-                delete eval_state->process_list;
-                eval_state->process_list = newlist;
-            }
-        }
-    }
-    if (step->next_sibling)
-        childByPath ((Step *) step->next_sibling, step->anyPath ());
+    return lst;
 }
 
 Sequence *Identifier::toSequence () const {
     Sequence *old = eval_state->process_list;
+
     Sequence *lst = new Sequence;
-    eval_state->process_list = lst;
     EvalState *es = eval_state;
-    if (!((Step *)first_child)->start_contextual)
+    if (!start_contextual)
         while (es->parent)
             es = es->parent;
     // TODO handle ..
     lst->append (new NodeValueItem (es->root));
-    childByPath ((Step *) first_child, false);
-    lst = eval_state->process_list;
+    for (AST *s = first_child; s; s = s->next_sibling) {
+        eval_state->process_list = lst;
+        lst = s->toSequence ();
+        delete eval_state->process_list;
+    }
+
+    eval_state->process_list = old;
+    return lst;
+}
+
+Sequence *PredicateFilter::toSequence () const {
+    Sequence *lst = first_child->toSequence ();
+    Sequence *old = eval_state->process_list;
+
+    for (AST *p = first_child->next_sibling; p; p = p->next_sibling) {
+        eval_state->process_list = lst;
+        lst = new Sequence;
+        for (NodeValueItem *itm = eval_state->process_list->first(); itm; itm = itm->nextSibling()) {
+            p->setRoot (itm->data);
+            if (p->toBool ())
+                lst->append (new NodeValueItem (itm->data));
+        }
+        delete eval_state->process_list;
+    }
+
     eval_state->process_list = old;
     return lst;
 }
@@ -756,7 +789,7 @@ bool StartsWith::toBool () const {
             if (s)
                 b = first_child->toString ().startsWith (s->toString ());
             else if (eval_state->parent)
-                b = eval_state->value ().startsWith (first_child->toString ());
+                b = eval_state->root.value ().startsWith (first_child->toString ());
         }
     }
     return b;
@@ -850,12 +883,14 @@ int Position::toInt () const {
         sequence = eval_state->sequence;
         if (eval_state->parent) {
             Sequence *lst = eval_state->parent->process_list;
-            Node *r = eval_state->root;
+            Node *r = eval_state->root.node;
+            String str = eval_state->root.string;
             if (lst) {
                 i = 0;
                 for (NodeValueItem *n = lst->first(); n; n = n->nextSibling()) {
                     i++;
-                    if (r == n->data.node)
+                    if ((r && r == n->data.node) ||
+                            (!r && str == n->data.string))
                         break;
                 }
             }
@@ -870,7 +905,7 @@ int StringLength::toInt () const {
         if (first_child)
             i = first_child->toString ().length ();
         else if (eval_state->parent)
-            i = eval_state->value ().length ();
+            i = eval_state->root.value ().length ();
         else
             i = 0;
     }
@@ -1016,7 +1051,7 @@ Sequence *Sort::toSequence () const {
     if (first_child) {
         Expression *exp = evaluateExpr (first_child->toString ());
         if (exp) {
-            exp->setRoot (eval_state->root);
+            exp->setRoot (eval_state->root.node);
             Sequence *lst = exp->toSequence ();
             if (lst->first () && first_child->next_sibling) {
                 Expression *sort_exp =
@@ -1033,6 +1068,54 @@ Sequence *Sort::toSequence () const {
     return AST::toSequence ();
 }
 
+Sequence *SubSequence::toSequence () const {
+    Sequence *lst = new Sequence;
+    AST *n = first_child;
+    if (n) {
+        Sequence *src = n->toSequence ();
+        n = n->next_sibling;
+        if (n) {
+            int p = n->toInt ();
+            if (p < 1)
+                p = 1;
+            int len = -1;
+            if (n->next_sibling)
+                len = n->next_sibling->toInt ();
+            NodeValueItem *itm = src->first ();
+            for (; itm && --p; itm = itm->nextSibling ())
+            {}
+            for (; itm && len--; itm = itm->nextSibling ())
+                lst->append (new NodeValueItem (itm->data));
+        }
+        delete src;
+    }
+    return lst;
+}
+
+Sequence *Tokenize::toSequence () const {
+    Sequence *lst = new Sequence;
+    if (first_child) {
+        if (first_child->next_sibling) {
+            String s = first_child->toString ();
+            GRegex *r = g_regex_new (
+                    (const char *) first_child->next_sibling->toString (),
+                    (GRegexCompileFlags) 0,
+                    (GRegexMatchFlags) 0, NULL);
+            GMatchInfo *match_info = NULL;
+            g_regex_match (r, (const char *)s, (GRegexMatchFlags)0, &match_info);
+            while (g_match_info_matches (match_info)) {
+                gchar *tok = g_match_info_fetch (match_info, 0);
+                lst->append (new NodeValueItem (String (tok)));
+                g_free (tok);
+                g_match_info_next (match_info, NULL);
+            }
+            g_match_info_free (match_info);
+            g_regex_unref (r);
+        }
+    }
+    return lst;
+}
+
 #define BIN_OP_TO_INT(NAME,OP)                                           \
     AST *second_child = first_child->next_sibling;                       \
     AST::Type t1 = first_child->type ();                                 \
@@ -1329,12 +1412,41 @@ static bool parseLiteral (const char *str, const char **end, AST *ast) {
     return false;
 }
 
+static bool parsePredicate (const char *str, const char **end, AST *ast) {
+    if (parseSpace (str, end))
+        str = *end;
+    if (!*str || *str++ != '[')
+        return false;
+    AST pred (new EvalState (ast->eval_state));
+    if (parseStatement (str, end, &pred)) {
+        str = *end;
+        if (parseSpace (str, end))
+            str = *end;
+        if (*str++ != ']')
+            return false;
+        if (pred.first_child) {
+            appendASTChild (ast, pred.first_child);
+            pred.first_child = NULL;
+        }
+    } else {
+        if (parseSpace (str, end))
+            str = *end;
+        if (*str++ != ']')
+            return false;
+    }
+    *end = str;
+    return true;
+}
+
 static bool parseStep (const char *str, const char **end, AST *ast) {
 #ifdef KMPLAYER_EXPR_DEBUG
     fprintf (stderr, "%s enter str:'%s'\n", __FUNCTION__, str);
 #endif
-    Step *entry = NULL;
+    AST *entry = NULL;
     const char *s = str;
+    bool recursive = *s == '/';
+    if (recursive)
+        s = ++str;
     if (*s == '.') {
         ++s;
         if (s && *s == '.') { // TODO
@@ -1343,8 +1455,6 @@ static bool parseStep (const char *str, const char **end, AST *ast) {
         } else {
             entry = new Step (ast->eval_state, true);
         }
-    } else if (*s == '/') {
-        entry = new Step (ast->eval_state);
     } else {
         bool is_attr = *s == '@';
         if (is_attr)
@@ -1358,52 +1468,33 @@ static bool parseStep (const char *str, const char **end, AST *ast) {
                 break;
         if (str == s)
             return false;
-        entry = new Step (ast->eval_state, str, s, is_attr);
-        while (*s == '[') {
-            AST pred (new EvalState (ast->eval_state));
-            if (parseStatement (s + 1, end, &pred)) {
-                str = *end;
-                if (parseSpace (str, end))
-                    str = *end;
-                if (*str == ']') {
-                    if (pred.first_child) {
-                        appendASTChild (entry, pred.first_child);
-                        pred.first_child = NULL;
-                    }
-                    s = ++str;
-                } else {
-                    delete entry;
-                    return false;
-                }
-            } else {
-                str = s + 1;
-                if (parseSpace (str, end))
-                    str = *end;
-                if (*str == ']') {
-                    s = ++str;
-                } else {
-                    delete entry;
-                    return false;
-                }
-            }
+        entry = new Step (ast->eval_state, str, s, is_attr, recursive);
+
+        AST fast (ast->eval_state);
+        const char *t = s;
+        if (parsePredicate (t, &t, &fast)) {
+            s = t;
+            while (parsePredicate (t, &t, &fast))
+                s = t;
+            entry->next_sibling = fast.first_child;
+            entry = new PredicateFilter (ast->eval_state, entry);
+            fast.first_child = NULL;
         }
+
     }
-    if (entry) {
-        appendASTChild (ast, entry);
-        *end = s;
+    appendASTChild (ast, entry);
+    *end = s;
 #ifdef KMPLAYER_EXPR_DEBUG
-        fprintf (stderr, "%s success end:'%s'\n", __FUNCTION__, *end);
+    fprintf (stderr, "%s success end:'%s'\n", __FUNCTION__, *end);
 #endif
-        return true;
-    }
-    return false;
+    return true;
 }
 
 static bool parseIdentifier (const char *str, const char **end, AST *ast) {
 #ifdef KMPLAYER_EXPR_DEBUG
     fprintf (stderr, "%s enter str:'%s'\n", __FUNCTION__, str);
 #endif
-    Identifier ident (ast->eval_state, NULL);
+    Identifier ident (ast->eval_state, NULL, false);
     bool has_any = false;
 
     if (parseSpace (str, end))
@@ -1416,11 +1507,11 @@ static bool parseIdentifier (const char *str, const char **end, AST *ast) {
     else if (!ast->eval_state->parent &&
             !ast->eval_state->def_root_tag.isEmpty ())
         appendASTChild (&ident, new Step (ast->eval_state,
-                    (const char *) ast->eval_state->def_root_tag, NULL));
+                    (const char *) ast->eval_state->def_root_tag, NULL,
+                    false, false));
     if (parseStep (str, end, &ident)) {
         str = *end;
         has_any = true;
-        ((Step *) ident.first_child)->start_contextual = start_contextual;
         if (*str == '/') {
             ++str;
             while (parseStep (str, end, &ident)) {
@@ -1433,7 +1524,7 @@ static bool parseIdentifier (const char *str, const char **end, AST *ast) {
     }
     *end = str;
     if (has_any) {
-        appendASTChild (ast, new Identifier (ast->eval_state, ident.first_child));
+        appendASTChild (ast, new Identifier (ast->eval_state, ident.first_child, start_contextual));
         ident.first_child = NULL;
 #ifdef KMPLAYER_EXPR_DEBUG
         fprintf (stderr, "%s success end:'%s'\n", __FUNCTION__, *end);
@@ -1531,10 +1622,14 @@ static bool parseFunction (const char *str, const char **end, AST *ast) {
                     func = new StringJoin (ast->eval_state);
                 else if (name == "string-length")
                     func = new StringLength (ast->eval_state);
+                else if (name == "subsequence")
+                    func = new SubSequence (ast->eval_state);
                 else if (name == "substring-after")
                     func = new SubstringAfter (ast->eval_state);
                 else if (name == "substring-before")
                     func = new SubstringBefore (ast->eval_state);
+                else if (name == "tokenize")
+                    func = new Tokenize (ast->eval_state);
                 else if (name == "escape-uri")
                     func = new EscapeUri (ast->eval_state);
                 else
@@ -1557,10 +1652,26 @@ static bool parseFactor (const char *str, const char **end, AST *ast) {
 #ifdef KMPLAYER_EXPR_DEBUG
     fprintf (stderr, "%s enter str:'%s'\n", __FUNCTION__, str);
 #endif
-    return parseGroup (str, end, ast) ||
-            parseFunction (str, end, ast) ||
-            parseLiteral (str, end, ast) ||
-            parseIdentifier (str, end, ast);
+    AST fast (ast->eval_state);
+    if (parseGroup (str, end, &fast) ||
+            parseFunction (str, end, &fast) ||
+            parseLiteral (str, end, &fast) ||
+            parseIdentifier (str, end, &fast)) {
+        str = *end;
+        const char *s = *end;
+        if (parsePredicate (s, &s, &fast)) {
+            *end = s;
+            while (parsePredicate (s, &s, &fast))
+                *end = s;
+            appendASTChild (ast,
+                    new PredicateFilter (ast->eval_state, fast.first_child));
+        } else {
+            appendASTChild (ast, fast.first_child);
+        }
+        fast.first_child = NULL;
+        return true;
+    }
+    return false;
 }
 
 struct Keyword {
