@@ -193,7 +193,7 @@ void Playlist::show (View *view)
             view->before = evaluateExpr (view->getAttribute ("before"));
         if (view->before) {
             view->before->setRoot (this);
-            NodeValueList *ls = view->before->toNodeList ();
+            Sequence *ls = view->before->toSequence ();
             if (ls->first())
                 before = ls->first()->data.node;
         }
@@ -207,7 +207,7 @@ void Playlist::show (View *view)
             groups = new GroupMap;
 
         view->show->setRoot (this);
-        NodeValueList *lst = view->show->toNodeList ();
+        Sequence *lst = view->show->toSequence ();
         for (NodeValueItem *itm = lst->first(); itm; itm = itm->nextSibling())
             if (itm->data.node->isElementNode ()) {
                 Element *e = static_cast <Element *> (itm->data.node);
