@@ -44,6 +44,7 @@ public:
     SizeType (const String & s, bool force_perc=false);
     void reset ();
     SizeType & operator = (const String & s);
+    SizeType & operator = (Single d);
     SizeType & operator += (const SizeType & s);
     SizeType & operator -= (const SizeType & s);
     SizeType & operator /= (const int i)
@@ -225,7 +226,7 @@ public:
     unsigned char text_writing;
     unsigned char text_align;
     unsigned char padding;
-    short font_size;
+    SizeType font_size;
 };
 
 class KMPLAYER_NO_EXPORT SmilColorProperty {
@@ -288,10 +289,12 @@ const short id_node_par = 111;
 const short id_node_seq = 112;
 const short id_node_switch = 113;
 const short id_node_excl = 114;
-const short id_node_text = 122;
-const short id_node_ref = 123;
-const short id_node_brush = 124;
-const short id_node_smil_text = 125;
+const short id_node_text = 120;
+const short id_node_ref = 121;
+const short id_node_brush = 122;
+const short id_node_smil_text = 123;
+const short id_node_tev = 124;
+const short id_node_clear = 125;
 const short id_node_text_styling = 126;
 const short id_node_set_value = 127;
 const short id_node_new_value = 128;
@@ -789,9 +792,10 @@ public:
 class KMPLAYER_NO_EXPORT Brush : public MediaType {
 public:
     Brush (NodePtr & d);
+    virtual void init ();
     virtual void accept (Visitor *);
     virtual void parseParam (const TrieString &, const String &);
-    unsigned int color;
+    SmilColorProperty color;
 };
 
 class KMPLAYER_NO_EXPORT SmilText : public Element {
@@ -812,6 +816,7 @@ public:
     virtual void accept (Visitor *v) { v->visit (this); }
 
     Surface *surface ();
+    void updateBounds (bool remove);
 
     SmilColorProperty background_color;
     MediaOpacity media_opacity;
@@ -819,6 +824,8 @@ public:
     SmilTextProperties props;
     SurfacePtrW text_surface;
     NodePtrW region_node;
+    CalculatedSizer sizes;
+    SSize size;
     ConnectionLink region_attach;
     ConnectionList media_attached;
     MouseListeners mouse_listeners;
@@ -841,6 +848,25 @@ public:
     String tag;
 };
 
+class KMPLAYER_NO_EXPORT TemporalMoment : public Element {
+public:
+    TemporalMoment (NodePtr &doc, short id, const String &tag);
+    ~TemporalMoment ();
+    virtual void init ();
+    virtual void activate ();
+    virtual void begin ();
+    virtual void deactivate ();
+    KDE_NO_EXPORT const char *nodeName () const { return (const char *)tag; }
+    Node *childFromTag (const String & tag);
+    virtual void parseParam (const TrieString &name, const String &value);
+    virtual void message (MessageType msg, void *content=NULL);
+    virtual void *role (RoleType msg, void *content=NULL);
+    virtual void accept (Visitor *v) { v->visit (this); }
+
+    Runtime *runtime;
+    String tag;
+};
+
 class KMPLAYER_NO_EXPORT StateValue : public Element {
 public:
     ~StateValue ();
