@@ -219,6 +219,42 @@ static gboolean findNode (PlayListData *pld, GtkTreeModel *model, Node *node, Gt
     return findNodeImpl (pld, model, node, iter, b);
 }
 
+static PlayListData *
+getListData (Application *app, GtkTreeModel *model, GtkTreeIter *iter) {
+    GValue value = { 0 , };
+    gtk_tree_model_get_value (model, iter, COL_ID, &value);
+    int id = g_value_get_int (&value);
+    g_value_unset (&value);
+    PlayListData * pld = CONTROL(app)->playlist_data;
+    while (pld && pld->id != id)
+        pld = pld->next;
+    return pld;
+}
+
+static PlayListData *getListData (Control *ctr, GtkTreePath *path) {
+    if (gtk_tree_path_get_depth (path) < 1)
+        return 0L;
+    gint id = gtk_tree_path_get_indices (path) [0];
+    PlayListData * pld = ctr->playlist_data;
+    while (pld && id-- > 0)
+        pld = pld->next;
+    return pld;
+}
+
+static PlayListData *getListDataFromSelection (Application *app, GtkTreePath **path = 0L) {
+    GtkTreeIter iter;
+    GtkTreeSelection *sel = gtk_tree_view_get_selection
+        (GTK_TREE_VIEW (app->playlist_view));
+    GtkTreeModel * model = gtk_tree_view_get_model
+        (GTK_TREE_VIEW (app->playlist_view));
+    if (gtk_tree_selection_get_selected (sel, 0L, &iter)) {
+        if (path)
+            *path = gtk_tree_model_get_path (model, &iter);
+        return getListData (app, model, &iter);
+    }
+    return 0L;
+}
+
 void KMPlayer::updatePlayTree (Application *app) {
     app->in_tree_update = true;
     PlayListData *pld = CONTROL(app)->playlist_data;
@@ -289,15 +325,29 @@ void KMPlayer::updateTree (Application * app, short id) {
 
     } else {
 
+        GtkTreePath *sel_path = 0L;
+        PlayListData *sel_pld = getListDataFromSelection (app, &sel_path);
+        if (sel_pld && sel_pld->id != id) {
+            gtk_tree_path_free (sel_path);
+            sel_path = NULL;
+        }
+
         valid_tree = gtk_tree_store_remove (GTK_TREE_STORE (model), &oldriter);
         if (valid_tree)
             gtk_tree_store_insert_before(GTK_TREE_STORE(model),&riter,0L,&oldriter);
         else
             gtk_tree_store_append (GTK_TREE_STORE(model), &riter, 0L);
+
         populate (GTK_TREE_STORE (model), pld,
                 pld->document.ptr (), &riter, 0L, &fpath);
 
-        while (!gtk_tree_model_get_iter (model, &valid_iter, fpath) &&
+        if (!fpath)
+            fpath = sel_path;
+        else if (sel_path)
+            gtk_tree_path_free (sel_path);
+
+        while (fpath &&
+                !gtk_tree_model_get_iter (model, &valid_iter, fpath) &&
                 gtk_tree_path_up (fpath));
     }
 
@@ -315,42 +365,6 @@ void KMPlayer::updateTree (Application * app, short id) {
     app->in_tree_update = false;
 }
 
-static PlayListData *
-getListData (Application * app, GtkTreeModel * model, GtkTreeIter *iter) {
-    GValue value = { 0 , };
-    gtk_tree_model_get_value (model, iter, COL_ID, &value);
-    int id = g_value_get_int (&value);
-    g_value_unset (&value);
-    PlayListData * pld = CONTROL(app)->playlist_data;
-    while (pld && pld->id != id)
-        pld = pld->next;
-    return pld;
-}
-
-static PlayListData *getListData (Control *ctr, GtkTreePath * path) {
-    if (gtk_tree_path_get_depth (path) < 1)
-        return 0L;
-    gint id = gtk_tree_path_get_indices (path) [0];
-    PlayListData * pld = ctr->playlist_data;
-    while (pld && id-- > 0)
-        pld = pld->next;
-    return pld;
-}
-
-static PlayListData * getListDataFromSelection (Application * app, GtkTreePath ** path = 0L) {
-    GtkTreeIter iter;
-    GtkTreeSelection *sel = gtk_tree_view_get_selection
-        (GTK_TREE_VIEW (app->playlist_view));
-    GtkTreeModel * model = gtk_tree_view_get_model
-        (GTK_TREE_VIEW (app->playlist_view));
-    if (gtk_tree_selection_get_selected (sel, 0L, &iter)) {
-        if (path)
-            *path = gtk_tree_model_get_path (model, &iter);
-        return getListData (app, model, &iter);
-    }
-    return 0L;
-}
-
 void collapseSelectedList (Application *app)
 {
     GtkTreeIter iter;
@@ -520,6 +534,7 @@ static void cb_selection_changed (GtkTreeSelection *selection, Application *app)
         for (PlayListData *pd = CONTROL(app)->playlist_data; pd; pd = pd->next)
             if (pd != pld && (pd->edit_mode || pd->show_all)) {
                 pd->show_all = false;
+                pd->current = NULL;
                 if (pd->edit_mode) {
                     setPlaylistEdit (app, pd, false);
                 } else {
@@ -539,7 +554,7 @@ static void cb_selection_changed (GtkTreeSelection *selection, Application *app)
                 mrl->document ()->m_tree_version)
             g_idle_add ((GSourceFunc) delayedUpdateTree, (gpointer) app);
         setInfoContent (app, node, pld->edit_mode);
-        if (mrl && pld->id > 0) //FIXME, hack to become less jumpy when changing trees
+        if (pld->id > 0) //FIXME, hack to become less jumpy when changing trees
             pld->current = node;
 
         if (CONTROL(app)->current_selected.ptr () != node) {
@@ -1030,15 +1045,15 @@ void cb_playlistTitle (Application *app) {
 void togglePlaylistShowAll (Application *app) {
     //g_print ("togglePlaylistShowAll\n");
     PlayListData * pld = getListDataFromSelection (app);
+    if (!pld) { // no selection, reset one that has show_all
+        for (PlayListData *pd = CONTROL(app)->playlist_data; pd; pd = pd->next)
+            if (pd->show_all) {
+                pld = pd;
+                break;
+            }
+    }
     if (pld) {
         bool selected = !pld->show_all;
-        if (!selected && !pld->show_all) { // click outside editable tree
-            for (PlayListData * pd = CONTROL(app)->playlist_data; pd; pd = pd->next)
-                if (pd->show_all) {
-                    pld = pd;
-                    break;
-                }
-        }
         pld->show_all = selected;
         pld->updated_document = NULL;
         g_idle_add ((GSourceFunc) delayedUpdateTree, (gpointer) app);
