@@ -41,6 +41,7 @@ static GtkWidget *xembed;
 static Window gtk_socket_id;
 static long parent_id;
 static gboolean plugin_embedded;
+static gboolean whole_screen;
 
 static void output (const char *format, ...) {
     static gboolean need_cr;
@@ -309,6 +310,7 @@ static gboolean initPlayer (void * p)
     ns_notify.getUrl = npGetUrl;
     ns_notify.evaluate = nsEvaluate;
     ns_notify.finishStream = npFinish;
+    gint width;
 
     nsdata = g_strdup ("");
 
@@ -327,8 +329,12 @@ static gboolean initPlayer (void * p)
         g_tree_insert (npp_plugins, (char *) mimetype, nplib);
     }
     nppInstanceWindowParent (npdata, (void *) gtk_socket_id);
+    if (whole_screen)
+       width = parent_id ? 800 : 565;
+    else
+       width = parent_id ? 640 : 565;
     nppInstanceWindowDimension (npdata,
-            0, 0, parent_id ? 640 : 565, parent_id ? 480 : 424);
+            0, 0, width, parent_id ? 480 : 424);
 
     if (!play ())
         quit ();
@@ -592,6 +598,8 @@ int main (int argc, char **argv) {
             mimetype = g_strdup (argv[i]);
         } else if (!strcmp (argv [i], "-wid") && ++i < argc) {
             parent_id = strtol (argv[i], 0L, 10);
+        } else  if (!strcmp (argv [i], "-max")) {
+           whole_screen = TRUE;
         } else if (!strcmp (argv [i], "-args")) {
             ++i;
             break;
@@ -600,7 +608,7 @@ int main (int argc, char **argv) {
         }
     }
     if (!(object_url && plugin)) {
-        fprintf (stderr, "Usage: %s -m mimetype -p plugin url <-wid id> <--args arg0=val0 [arg1=val1 ..]>\n", argv[0]);
+        fprintf (stderr, "Usage: %s -m mimetype -p plugin url <-wid id> <-max> <--args arg0=val0 [arg1=val1 ..]>\n", argv[0]);
         return 1;
     }
 
@@ -651,7 +659,8 @@ int main (int argc, char **argv) {
             GTK_SIGNAL_FUNC (configureEvent), NULL);
 
     align = gtk_alignment_new (0, 0, 1, 1);
-    gtk_alignment_set_padding (GTK_ALIGNMENT (align), 0, 0, 80, 80);
+    if (!whole_screen)
+        gtk_alignment_set_padding (GTK_ALIGNMENT (align), 0, 0, 80, 80);
     xembed = gtk_socket_new();
     g_signal_connect (G_OBJECT (xembed), "plug-added",
             GTK_SIGNAL_FUNC (pluginAdded), NULL);
