@@ -909,16 +909,23 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Brush * brush) {
             cairo_rectangle (cr, clip_rect.x (), clip_rect.y (),
                     clip_rect.width (), clip_rect.height ());
         }
+        unsigned int color = brush->color.color;
+        if (!color) {
+            color = brush->background_color.color;
+            opacity *= brush->background_color.opacity / 100.0;
+        } else {
+            opacity *= brush->color.opacity / 100.0;
+        }
         opacity *= brush->media_opacity.opacity / 100.0;
         if (opacity < 0.99) {
             cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
             cairo_set_source_rgba (cr,
-                    1.0 * ((brush->color >> 16) & 0xff) / 255,
-                    1.0 * ((brush->color >> 8) & 0xff) / 255,
-                    1.0 * (brush->color & 0xff) / 255,
+                    1.0 * ((color >> 16) & 0xff) / 255,
+                    1.0 * ((color >> 8) & 0xff) / 255,
+                    1.0 * (color & 0xff) / 255,
                     opacity);
         } else {
-            CAIRO_SET_SOURCE_RGB (cr, brush->color);
+            CAIRO_SET_SOURCE_RGB (cr, color);
         }
         cairo_fill (cr);
         if (opacity < 0.99)
@@ -963,6 +970,7 @@ public:
     using Visitor::visit;
     void visit (TextNode *);
     void visit (SMIL::TextFlow *);
+    void visit (SMIL::TemporalMoment *);
 
     void addRichText (const String &txt);
     void push ();
@@ -981,8 +989,8 @@ public:
 void SmilTextInfo::span (float scale) {
     char buf[64];
     String s = "<span";
-    if (props.font_size > -1)
-        s += String (" size='") + String::number ((int)(1024 * scale * props.font_size)) + "'";
+    if (props.font_size.size () > -1)
+        s += String (" size='") + String::number ((int)(1024 * scale * props.font_size.size ())) + "'";
     s += String (" face='") + props.font_family + "'";
     if (props.font_color > -1) {
         snprintf (buf, sizeof (buf), " foreground='#%06x'", props.font_color);
@@ -1032,7 +1040,7 @@ void SmilTextVisitor::addRichText (const String &txt) {
 void SmilTextVisitor::push () {
     if (!rich_text.isEmpty ()) {
         int pxw, pxh;
-        float fs = info.props.font_size;
+        float fs = info.props.font_size.size ();
         if (fs < 0)
             fs = TextActor::defaultFontSize ();
         float maxfs = max_font_size;
@@ -1081,11 +1089,11 @@ void SmilTextVisitor::visit (TextNode *text) {
 }
 
 void SmilTextVisitor::visit (SMIL::TextFlow *flow) {
-    if (flow->firstChild ()) {
-        bool new_block = SMIL::id_node_p == flow->id ||
-            SMIL::id_node_br == flow->id ||
-            SMIL::id_node_div == flow->id;
-        float fs = info.props.font_size;
+    bool new_block = SMIL::id_node_p == flow->id ||
+        SMIL::id_node_br == flow->id ||
+        SMIL::id_node_div == flow->id;
+    if ((new_block && !rich_text.isEmpty ()) || flow->firstChild ()) {
+        float fs = info.props.font_size.size ();
         if (fs < 0)
             fs = TextActor::defaultFontSize ();
         int par_extra = SMIL::id_node_p == flow->id
@@ -1097,15 +1105,16 @@ void SmilTextVisitor::visit (SMIL::TextFlow *flow) {
             push ();
 
         info.props.mask (flow->props);
-        if (info.props.font_size > max_font_size)
-            max_font_size = info.props.font_size;
+        if ((float)info.props.font_size.size () > max_font_size)
+            max_font_size = info.props.font_size.size ();
         info.span (scale);
 
-        flow->firstChild ()->accept (this);
+        if (flow->firstChild ())
+            flow->firstChild ()->accept (this);
 
         if (rich_text.isEmpty ())
             par_extra = 0;
-        if (new_block)
+        if (new_block && flow->firstChild ())
             push ();
         voffset += par_extra;
 
@@ -1115,6 +1124,12 @@ void SmilTextVisitor::visit (SMIL::TextFlow *flow) {
         flow->nextSibling ()->accept (this);
 }
 
+void SmilTextVisitor::visit (SMIL::TemporalMoment *tm) {
+    if (tm->state >= Node::state_began
+            && tm->nextSibling ())
+        tm->nextSibling ()->accept (this);
+}
+
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::SmilText *txt) {
     Surface *s = txt->surface ();
     if (!s)
@@ -1129,8 +1144,16 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::SmilText *txt) {
         float scale = 1.0 * w / (double)s->bounds.width ();
         SmilTextVisitor info (w, scale, txt->props);
 
-        if (txt->firstChild ())
-            txt->firstChild ()->accept (&info);
+        Node *first = txt->firstChild ();
+        for (Node *n = first; n; n = n->nextSibling ())
+            if (SMIL::id_node_clear == n->id) {
+                if (n->state >= Node::state_began)
+                    first = n->nextSibling ();
+                else
+                    break;
+            }
+        if (first)
+            first->accept (&info);
 
         info.push ();
         if (info.first) {
@@ -1138,10 +1161,10 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::SmilText *txt) {
 
             CAIRO_SET_SOURCE_RGB (cr_txt, 0);
             SmilTextBlock *b = info.first;
+            int hoff = 0;
             int voff = 0;
             while (b) {
-                cairo_translate (cr_txt, 0, b->rect.y() - voff);
-                voff = b->rect.y ();
+                cairo_translate (cr_txt, b->rect.x() - hoff, b->rect.y() - voff);
                 PangoLayout *layout = pango_cairo_create_layout (cr_txt);
                 pango_layout_set_width (layout, 1.0 * w * PANGO_SCALE);
                 pango_layout_set_wrap (layout, PANGO_WRAP_WORD_CHAR);
@@ -1154,6 +1177,8 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::SmilText *txt) {
 
                 g_object_unref (layout);
 
+                hoff = b->rect.x ();
+                voff = b->rect.y ();
                 SmilTextBlock *tmp = b;
                 b = b->next;
                 delete tmp;
@@ -1162,6 +1187,8 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::SmilText *txt) {
 
             // update bounds rect
             s->bounds = matrix.toUser(IRect(scr.point, ISize(w, info.voffset)));
+            txt->size = s->bounds.size;
+            txt->updateBounds (false);
 
             // update coord. for painting below
             scr = matrix.toScreen (s->bounds);
@@ -1671,7 +1698,6 @@ void MouseVisitor::surfaceEvent (Node *node, Surface *s) {
     int rx = scr.x(), ry = scr.y(), rw = scr.width(), rh = scr.height();
     const bool inside = x > rx && x < rx+rw && y > ry && y< ry+rh;
     const bool had_mouse = s->has_mouse;
-    s->has_mouse = inside;
     if (deliverAndForward (node, s, inside, true) &&
             (inside || s->has_mouse) &&
             s->firstChild () && s->firstChild ()->node) {
