@@ -291,6 +291,56 @@ int main() {
         TEST_VERIFY (expr->toInt () == 10);
         delete expr;
     }
+    {
+        TEST_INFO (Test tokenize(/books/book[2]/title,'\\S+') == 3);
+        Expression *expr = evaluateExpr ("tokenize(/books/book[2]/title,'\\S+')");
+        expr->setRoot (books);
+        TEST_VERIFY (expr->toInt () == 3);
+        delete expr;
+    }
+    {
+        TEST_INFO (Test tokenize(/books/book[2]/title,'\\S+')[1] == Uncle);
+        Expression *expr = evaluateExpr ("tokenize(/books/book[2]/title,'\\S+')[1]");
+        expr->setRoot (books);
+        TEST_VERIFY (expr->toString () == "Uncle");
+        delete expr;
+    }
+    {
+        TEST_INFO (Test tokenize(/books/book[2]/title,'\\S+')[position() = 1] == Uncle);
+        Expression *expr = evaluateExpr ("tokenize(/books/book[2]/title,'\\S+')[position() = 1]");
+        expr->setRoot (books);
+        debugLog() << expr->toString() << endl;
+        TEST_VERIFY (expr->toString () == "Uncle");
+        delete expr;
+    }
+    {
+        TEST_INFO (Test subsequence(tokenize(/books/book[2]/title,'\\S+'),1,1) == Uncle);
+        Expression *expr = evaluateExpr ("subsequence(tokenize(/books/book[2]/title,'\\S+'),1,1)");
+        expr->setRoot (books);
+        TEST_VERIFY (expr->toString () == "Uncle");
+        delete expr;
+    }
+    {
+        TEST_INFO (Test count(subsequence(tokenize(/books/book[2]/title,'\\S+'),3,0)) == '');
+        Expression *expr = evaluateExpr ("count(subsequence(tokenize(/books/book[2]/title,'\\S+'),3,0))");
+        expr->setRoot (books);
+        TEST_VERIFY (expr->toInt () == 0);
+        delete expr;
+    }
+    {
+        TEST_INFO (Test subsequence(tokenize(/books/book[2]/title,'\\S+'),3,-1) == Uncle);
+        Expression *expr = evaluateExpr ("subsequence(tokenize(/books/book[2]/title,'\\S+'),3,-1)");
+        expr->setRoot (books);
+        TEST_VERIFY (expr->toString () == "Cabin");
+        delete expr;
+    }
+    {
+        TEST_INFO (Test string-join(subsequence(tokenize(/books/book[2]/title,'\\S+'),2),',') == 'Toms,Cabin');
+        Expression *expr = evaluateExpr ("string-join(subsequence(tokenize(/books/book[2]/title,'\\S+'),2),',')");
+        expr->setRoot (books);
+        TEST_VERIFY (expr->toString () == "Tom's,Cabin");
+        delete expr;
+    }
     books->document ()->dispose ();
     return 0;
 }
