@@ -36,6 +36,16 @@ int main() {
         TEST_VERIFY (String ("    r ").stripWhiteSpace () == "r" );
         TEST_VERIFY (String ("   f   r  ").stripWhiteSpace () == "f   r" );
     }
+    {
+        TEST_INFO(Test indexOf in "Im S\xc3\xbc""den zahlt man 10\xe2\x82\xac");
+        String s = "Im S\xc3\xbc""den zahlt man 10\xe2\x82\xac";
+        TEST_VERIFY (s.indexOf ("Im") == 0);
+        TEST_VERIFY (s.indexOf ("\xc3\xbc""d") == 4);
+        TEST_VERIFY (s.indexOf ("zahlt") == 9);
+        TEST_VERIFY (s.indexOf ("\xe2\x82\xac") == 21);
+        TEST_VERIFY (s.indexOf ("\xe2\x82\xac", 6) == 21);
+        fprintf(stderr, "%s\n", (const char *)s);
+    }
     {
         TEST_INFO (Test TextStream on 'foo');
         String foo ("foo");
@@ -89,5 +99,18 @@ int main() {
         TextStream out;
         TEST_VERIFY (out.release().isEmpty());
     }
+    {
+        TEST_INFO (Test indes/pos/left/mid/truncate in Cyrillic path);
+        String s = "MyDocs/Cyrillic\xd0\x90\xd0\x91\xd0\x92/\xd1\xa0""123\xd1\xac"".mp3";
+        int p = s.lastIndexOf (Char ('/'));
+        TEST_VERIFY (s.length() == 28);
+        TEST_VERIFY (s.lastIndexOf (Char ('/')) == 18);
+        TEST_VERIFY (s.indexOf (Char ('/'), 8) == 18);
+        TEST_VERIFY (s.left (p).length () == 18);
+        TEST_VERIFY (s.mid (p).length () == 10);
+        TEST_VERIFY (s == s.left (p) + s.mid (p));
+        s.truncate (p);
+        TEST_VERIFY (s.length() == 18);
+    }
     return 0;
 }
